/*
 *  ISS Pointer - Runs a Stepper and Servo to move in AZIMUTH and ALTITUDE 
 *  When used in conjunction with iss_tracker.py on a Raspberry Pi, it will
 *  point to the International Space Station as it flys overhead.
 *  
 *  This sketch can also be used to point to anything in above the horizon Azimuth/Altitude
 *  Azimuth = -200 to 200 horizontal steps 
 *  Altitude= 0 to 90 degrees angle
 *  
 *  This sketch sets up a web server on port 80 which controls a Stepper motor and a Servo
 *  The web server will control the Adafruit TB6612 Driver board depending on the request
 *  See http://server_ip/ for usage (or see docs for more info)
 *  The server_ip is the IP address of the ESP8266 module which will be 
 *  printed to Serial Monitor port.
 *  When turned on, the internal LED blinks when the module is connected to your WiFi.
 *  
 *  The features are controlled by externally calling the HTTP URLs below:
 *  
 * Stepper usage:
 *  http://{ip_address}/stepper/stop
 *  http://{ip_address}/stepper/start
 *  http://{ip_address}/stepper/rpm?[1 to 60]
 *  http://{ip_address}/stepper/steps?[-200 to 200]
 * 
 * Servo usage:
 *  http://{ip_address}/servo/value?[0 to 90]
 *  
 * External LED usage:
 *  http://{ip_address}/led/on
 *  http://{ip_address}/led/off
 *  
 *  Note: This controller has no security, so don't expose to Internet unless you
 *  don't mind someone playing with your servos! 
 *  
 *  Version 1.1 2015.12.13 R. Grokett
 *  - Initial 
 */

#include <ESP8266WiFi.h>
#include <Stepper.h>
#include <Servo.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define OLED_RESET 0  // GPIO0
Adafruit_SSD1306 display(OLED_RESET);

// -- USER EDIT -- 
const char* ssid     = "sa=DE78";  // YOUR WIFI SSID
const char* password = "8135419783651983";  // YOUR WIFI PASSWORD


// change this to your motor if not NEMA-17 200 step
#define STEPS 2048  // Max steps for one revolution
#define RPM 10     // Max RPM
#define DELAY 1    // Delay to allow Wifi to work

#define SERVER_TIMEOUT 10000 // Timeout before server disconnects a client
// -- END USER EDIT --

int SVRO = D3;    // GPIO 15 Servo Control
int STBY = D5;     // GPIO 5 TB6612 Standby
int LEDIN = D4;    // GPIO 0 (built-in LED)
int LEDEX = D8;    // GPIO 4 External LED

// GPIO Pins for Motor Driver board
Stepper stepper(STEPS, D5, D7, D6, D0);

// Create an instance of the server
// specify the port to listen on as an argument
WiFiServer server(80);

Servo myservo;

// Initialize 
void setup() {
  Serial.begin(115200);
  delay(10);
  
  // display
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3C (for the 64x48)
  display.display();

  // prepare onboard LED
  pinMode(LEDIN, OUTPUT);
  digitalWrite(LEDIN, LOW);

  // prepare external LED
  pinMode(LEDEX, OUTPUT);
  digitalWrite(LEDEX, HIGH);

  // prepare STBY GPIO and set Stepper motors
  pinMode(STBY, OUTPUT);
  digitalWrite(STBY, HIGH);
  
  // Set default speed to Max (doesn't move motor)
  stepper.setSpeed(RPM);


  // Connect to WiFi network
  Serial.println();
  Serial.println();
  printMillis();
  Serial.print("ISSPointer: Connecting to ");
  Serial.println(ssid);


  WiFi.disconnect();
  WiFi.persistent(false);
  WiFi.mode(WIFI_STA);
//  WiFi.config(ip, gateway, subnet);
  WiFi.begin(ssid, password);
  
  while (WiFi.status() != WL_CONNECTED) {
//    delay(1);
    SleepMilliSeconds(100);
    Serial.print(".");
  }
  Serial.println("");
  printMillis();
  Serial.println("WiFi connected");
  
  // Start the server
  server.begin();
  printMillis();
  Serial.println("Server started");

  // Print the IP address
  printMillis();
  Serial.println(WiFi.localIP());

  // Blink onboard LED to signify its connected
  blink();
  blink();
  blink();
  blink();

  // prepare Servo GPIO
  myservo.attach(SVRO);
  myservo.write(0);
}

void loop() {
  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) {
    return;
  }

  String respMsg = "";    // HTTP Response Message
  
  // Wait until the client sends some data
  printMillis();
  Serial.println("new client");
  
  int timeout = millis() + SERVER_TIMEOUT;
  while(!client.available()){
//    delay(1);
    SleepMilliSeconds(1);
    yield();
    if(millis() > timeout)
    {
      printMillis();
     Serial.println("Timeout reached. Disconnecting client.");
     client.stop();
     return;
    }
  }
  
  // Read the first line of the request
 // String req = client.readStringUntil('\r');
  String req = client.readStringUntil('.');
  Serial.println((req).substring(5, 26)); //complete
  Serial.println((req).substring(13, 15)); //day
  Serial.println((req).substring(10, 12)); //month
  Serial.println((req).substring(5, 9)); //year
  Serial.println((req).substring(16, 21)); //time
  
  //Serial.print(".");
  printMillis();
  Serial.println(req);
  client.flush();

  //display
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(5,0);
  //line1 nextpass
  display.print("Next Pass");
  display.drawFastHLine(0, 10, 63, WHITE); 
  //line2 date
  display.setCursor(2,14);
  display.print((req).substring(13, 15)); //day
  display.print(".");
  display.print((req).substring(10, 12)); //month
  display.print(".");
  display.print((req).substring(5, 9)); //year
  display.drawFastHLine(0, 24, 63, WHITE); 
  //line3 time 
  display.setCursor(6,28);
  display.print((req).substring(16, 21));
  display.print(" Uhr");
  //I
  display.drawFastHLine(22, 41, 5, WHITE);
  display.drawFastHLine(24, 42, 1, WHITE);
  display.drawFastHLine(23, 43, 1, WHITE);
  display.drawFastHLine(22, 44, 1, WHITE);
  display.drawFastHLine(21, 45, 1, WHITE);
  display.drawFastHLine(20, 46, 1, WHITE);
  display.drawFastHLine(18, 47, 5, WHITE);  
  //S
  display.drawFastHLine(32, 41, 6, WHITE);
  display.drawFastHLine(31, 42, 1, WHITE);
  display.drawFastHLine(30, 43, 1, WHITE);
  display.drawFastHLine(29, 44, 6, WHITE);
  display.drawFastHLine(34, 45, 1, WHITE);
  display.drawFastHLine(33, 46, 1, WHITE);
  display.drawFastHLine(27, 47, 6, WHITE);
  //S
  display.drawFastHLine(42, 41, 6, WHITE);
  display.drawFastHLine(41, 42, 1, WHITE);
  display.drawFastHLine(40, 43, 1, WHITE);
  display.drawFastHLine(39, 44, 6, WHITE);
  display.drawFastHLine(44, 45, 1, WHITE);
  display.drawFastHLine(43, 46, 1, WHITE);
  display.drawFastHLine(37, 47, 6, WHITE);
  //showdisplay
  display.display();
  
  // CONTROL STEPPER
  if (req.indexOf("/led/off") != -1) {
    digitalWrite(LEDEX, HIGH);
    respMsg = "OK: LED OFF";
  } 
  else if (req.indexOf("/led/on") != -1) {
    digitalWrite(LEDEX, LOW);
    respMsg = "OK: LED ON";
  } 
  // CONTROL SERVO 
  else if (req.indexOf("/servo/value") != -1) {
    int az = getValue(req);
    if ((az < 0) || (az > 90)) {
      respMsg = "ERROR: servo out of range 0 to 90";
    } else {      
      myservo.write(az);
      respMsg = "OK: ALTITUDE = "+String(az);
    }
  }
  // CONTROL STEPPER
  else if (req.indexOf("/stepper/stop") != -1) {
    digitalWrite(STBY, LOW);
    respMsg = "OK: MOTORS OFF";
  } 
  else if (req.indexOf("/stepper/start") != -1) {
    digitalWrite(STBY, HIGH);
    blink();
    respMsg = "OK: MOTORS ON";
  } 
  else if (req.indexOf("/stepper/rpm") != -1) {
    int rpm = getValue(req);
    if ((rpm < 1) || (rpm > RPM)) {
      respMsg = "ERROR: rpm out of range 1 to "+ String(RPM);
    } else {
      stepper.setSpeed(rpm);
      respMsg = "OK: RPM = "+String(rpm);
    }
  }
  // This is just a simplistic method of handling + or - number steps...
  else if (req.indexOf("/stepper/steps") != -1) {
    int steps = getValue(req);
    if ((steps == 0) || (steps < 0 - STEPS) || ( steps > STEPS )) {
      respMsg = "ERROR: steps out of range ";
    } else {  
      digitalWrite(STBY, HIGH);       // Make sure motor is on
      respMsg = "OK: STEPS = "+String(steps);
//      delay(DELAY); 
      SleepMilliSeconds(DELAY); 
      if ( steps > 0) { // Forward
        for (int i=0;i<steps;i++) {   // This loop is needed to allow Wifi to not be blocked by step
          stepper.step(-1);
//          delay(DELAY);
          SleepMilliSeconds(DELAY);
          yield();   
        }
      } else {         // Reverse
          for (int i=0;i>steps;i--) {   // This loop is needed to allow Wifi to not be blocked by step
            stepper.step(1);
//            delay(DELAY);
            SleepMilliSeconds(DELAY);
            yield(); 
          }  
      }
    }
  }
  else {
    respMsg = printUsage();
  }
    
  client.flush();

  // Prepare the response
  String s = "HTTP/1.1 200 OK\r\nContent-Type: text/plain\r\n\r\n";
  if (respMsg.length() > 0)
    s += respMsg;
  else
    s += "OK";
     
  s += "\n";
  s += "Free Heap: ";
  s += ESP.getFreeHeap();
  s += " Byte";
  s += "\n";

  // Send the response to the client
  client.print(s);
//  delay(10);
  SleepMilliSeconds(10);
  printMillis();
  Serial.println("Client disconnected");

  // The client will actually be disconnected 
  // when the function returns and 'client' object is detroyed
}

void printMillis()
{
  Serial.print("@");
  Serial.print(millis());
  Serial.print(": ");
}

int getValue(String req) {
  int val_start = req.indexOf('?');
  int val_end   = req.indexOf(' ', val_start + 1);
  if (val_start == -1 || val_end == -1) {
    Serial.print("Invalid request: ");
    Serial.println(req);
    return(0);
  }
  req = req.substring(val_start + 1, val_end);
  Serial.print("Request: ");
  Serial.println(req);
   
  return(req.toInt());
}

String printUsage() {
  // Prepare the usage response
  String s = "Stepper usage:\n";
  s += "http://{ip_address}/stepper/stop\n";
  s += "http://{ip_address}/stepper/start\n";
  s += "http://{ip_address}/stepper/rpm?[1 to " + String(RPM) + "]\n";
  s += "http://{ip_address}/stepper/steps?[-" + String(STEPS) + " to " + String(STEPS) +"]\n";
  s += "\n";
  s += "Servo usage:\n";
  s += "http://{ip_address}/servo/value?[0 to 90]\n";
  s += "\n";
  s += "LED usage:\n";
  s += "http://{ip_address}/led/on\n";
  s += "http://{ip_address}/led/off\n"; 
  return(s);
}

void blink() {
  digitalWrite(LEDIN, LOW);
  delay(100); 
  digitalWrite(LEDIN, HIGH);
  delay(100);
}

void SleepSeconds(unsigned int seconds)
{
  SleepMilliSeconds(seconds * 1000);
}

void SleepMilliSeconds(unsigned int milliseconds)
{
  unsigned long targetMillis = millis() + milliseconds;

  while(millis() < targetMillis)
  {
    yield();
  }
}
